@@ -28,6 +28,125 @@
 #include <libxml/xmlerror.h>
 #include <libxml/globals.h>
 
+/**
+ * xmlGetNodePath:
+ * @node: a node
+ *
+ * Build a structure based Path for the given node
+ *
+ * Returns the new path or NULL in case of error. The caller must free
+ *     the returned string
+ */
+xmlChar *
+xmlGetNodePath(xmlNodePtr node)
+{
+    xmlNodePtr cur, tmp, next;
+    xmlChar *buffer = NULL, *temp;
+    size_t buf_len;
+    xmlChar *buf;
+    char sep;
+    const char *name;
+    char nametemp[100];
+    int occur = 0;
+
+    if (node == NULL)
+        return (NULL);
+
+    buf_len = 500;
+    buffer = (xmlChar *) xmlMalloc(buf_len * sizeof(xmlChar));
+    if (buffer == NULL)
+        return (NULL);
+    buf = (xmlChar *) xmlMalloc(buf_len * sizeof(xmlChar));
+    if (buf == NULL) {
+        xmlFree(buffer);
+        return (NULL);
+    }
+
+    buffer[0] = 0;
+    cur = node;
+    do {
+        name = "";
+        sep = '?';
+        occur = 0;
+        if ((cur->type == XML_DOCUMENT_NODE) ||
+            (cur->type == XML_HTML_DOCUMENT_NODE)) {
+            if (buffer[0] == '/')
+                break;
+            sep = '/';
+            next = NULL;
+        } else if (cur->type == XML_ELEMENT_NODE) {
+            sep = '/';
+            name = (const char *) cur->name;
+            if (cur->ns) {
+                snprintf(nametemp, sizeof(nametemp) - 1,
+                         "%s:%s", cur->ns->prefix, cur->name);
+                nametemp[sizeof(nametemp) - 1] = 0;
+                name = nametemp;
+            }
+            next = cur->parent;
+
+            /*
+             * Thumbler index computation
+             */
+            tmp = cur->prev;
+            while (tmp != NULL) {
+                if (xmlStrEqual(cur->name, tmp->name))
+                    occur++;
+                tmp = tmp->prev;
+            }
+            if (occur == 0) {
+                tmp = cur->next;
+                while (tmp != NULL) {
+                    if (xmlStrEqual(cur->name, tmp->name))
+                        occur++;
+                    tmp = tmp->next;
+                }
+                if (occur != 0)
+                    occur = 1;
+            } else
+                occur++;
+        } else if (cur->type == XML_ATTRIBUTE_NODE) {
+            sep = '@';
+            name = (const char *) (((xmlAttrPtr) cur)->name);
+            next = ((xmlAttrPtr) cur)->parent;
+        } else {
+            next = cur->parent;
+        }
+
+        /*
+         * Make sure there is enough room
+         */
+        if (xmlStrlen(buffer) + sizeof(nametemp) + 20 > buf_len) {
+            buf_len =
+                2 * buf_len + xmlStrlen(buffer) + sizeof(nametemp) + 20;
+            temp = (xmlChar *) xmlRealloc(buffer, buf_len);
+            if (temp == NULL) {
+                xmlFree(buf);
+                xmlFree(buffer);
+                return (NULL);
+            }
+            buffer = temp;
+            temp = (xmlChar *) xmlRealloc(buf, buf_len);
+            if (temp == NULL) {
+                xmlFree(buf);
+                xmlFree(buffer);
+                return (NULL);
+            }
+            buf = temp;
+        }
+        if (occur == 0)
+            snprintf((char *) buf, buf_len, "%c%s%s",
+                     sep, name, (char *) buffer);
+        else
+            snprintf((char *) buf, buf_len, "%c%s[%d]%s",
+                     sep, name, occur, (char *) buffer);
+        snprintf((char *) buffer, buf_len, "%s", buf);
+        cur = next;
+    } while (cur != NULL);
+    xmlFree(buf);
+    return (buffer);
+}
+
 /**
  * xmlDebugDumpString:
  * @output:  the FILE * for the output
@@ -1801,81 +1920,27 @@ int
 xmlShellPwd(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED, char *buffer,
             xmlNodePtr node, xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
-    xmlNodePtr cur, tmp, next;
-    char buf[500];
-    char sep;
-    const char *name;
-    char nametemp[100];
-    int occur = 0;
+    xmlChar *path;
 
-    buffer[0] = 0;
     if (node == NULL)
         return (-1);
-    cur = node;
-    do {
-        name = "";
-        sep = '?';
-        occur = 0;
-        if ((cur->type == XML_DOCUMENT_NODE) ||
-            (cur->type == XML_HTML_DOCUMENT_NODE)) {
-            sep = '/';
-            next = NULL;
-        } else if (cur->type == XML_ELEMENT_NODE) {
-            sep = '/';
-            name = (const char *) cur->name;
-            if (cur->ns) {
-                snprintf(nametemp, 99, "%s:%s", cur->ns->prefix,
-                         cur->name);
-                name = nametemp;
-            }
-            next = cur->parent;
 
-            /*
-             * Thumbler index computation
-             */
-            tmp = cur->prev;
-            while (tmp != NULL) {
-                if (xmlStrEqual(cur->name, tmp->name))
-                    occur++;
-                tmp = tmp->prev;
-            }
-            if (occur == 0) {
-                tmp = cur->next;
-                while (tmp != NULL) {
-                    if (xmlStrEqual(cur->name, tmp->name))
-                        occur++;
-                    tmp = tmp->next;
-                }
-                if (occur != 0)
-                    occur = 1;
-            } else
-                occur++;
-        } else if (cur->type == XML_ATTRIBUTE_NODE) {
-            sep = '@';
-            name = (const char *) (((xmlAttrPtr) cur)->name);
-            next = ((xmlAttrPtr) cur)->parent;
-        } else {
-            next = cur->parent;
-        }
-        if (occur == 0)
-            snprintf(buf, sizeof(buf), "%c%s%s", sep, name, buffer);
-        else
-            snprintf(buf, sizeof(buf), "%c%s[%d]%s",
-                     sep, name, occur, buffer);
-        buf[sizeof(buf) - 1] = 0;
-        /*
-         * This test prevents buffer overflow, because this routine
-         * is only called by xmlShell, in which the second argument is
-         * 500 chars long.
-         * It is a dirty hack before a cleaner solution is found.
-         * Documentation should mention that the second argument must
-         * be at least 500 chars long, and could be stripped if too long.
-         */
-        if (strlen(buffer) + strlen(buf) > 499)
-            break;
-        strcpy(buffer, buf);
-        cur = next;
-    } while (cur != NULL);
+    path = xmlGetNodePath(node);
+    if (path == NULL)
+	return (-1);
+
+    /*
+     * This test prevents buffer overflow, because this routine
+     * is only called by xmlShell, in which the second argument is
+     * 500 chars long.
+     * It is a dirty hack before a cleaner solution is found.
+     * Documentation should mention that the second argument must
+     * be at least 500 chars long, and could be stripped if too long.
+     */
+    snprintf(buffer, 499, "%s", path);
+    buffer[499] = '0';
+    xmlFree(path);
+
     return (0);
 }
 
@@ -1987,7 +2052,24 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
             break;
         if (!strcmp(command, "bye"))
             break;
-        if (!strcmp(command, "validate")) {
+		if (!strcmp(command, "help")) {
+		  printf("\tbase         display XML base of the node\n");
+		  printf("\tbye          leave shell\n");
+		  printf("\tcat [node]   display node or current node\n");
+		  printf("\tcd [path]    change directory to path or to root\n");
+		  printf("\tdir [path]   dumps informations about the node (namespace, attributes, content)\n");
+		  printf("\tdu [path]    show the structure of the subtree under path or the current node\n");
+		  printf("\texit         leave shell\n");
+		  printf("\thelp         display this help\n");
+		  printf("\tfree         display memory usage\n");
+		  printf("\tload [name]  load a new document with name\n");
+		  printf("\tls [path]    list contents of path or the current directory\n");
+		  printf("\tpwd          display current working directory\n");
+		  printf("\tquit         leave shell\n");
+		  printf("\tsave [name]  save this document to name or the original name\n");
+		  printf("\tvalidate     check the document for errors\n");
+		  printf("\twrite [name] write the current node to the filename\n");
+        } else if (!strcmp(command, "validate")) {
             xmlShellValidate(ctxt, arg, NULL, NULL);
         } else if (!strcmp(command, "load")) {
             xmlShellLoad(ctxt, arg, NULL, NULL);
@@ -2038,6 +2120,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
                         case XPATH_NODESET:{
                                 int indx;
 
+				if (list->nodesetval == NULL)
+				    break;
+
                                 for (indx = 0;
                                      indx < list->nodesetval->nodeNr;
                                      indx++) {
@@ -2112,13 +2197,18 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
                                             "%s: no such node\n", arg);
                             break;
                         case XPATH_NODESET:
-                            if (list->nodesetval->nodeNr == 1) {
-                                ctxt->node = list->nodesetval->nodeTab[0];
+                            if (list->nodesetval != NULL) {
+				if (list->nodesetval->nodeNr == 1) {
+				    ctxt->node = list->nodesetval->nodeTab[0];
+				} else
+				    xmlGenericError(xmlGenericErrorContext,
+						    "%s is a %d Node Set\n",
+						    arg,
+						    list->nodesetval->nodeNr);
                             } else
                                 xmlGenericError(xmlGenericErrorContext,
-                                                "%s is a %d Node Set\n",
-                                                arg,
-                                                list->nodesetval->nodeNr);
+                                                "%s is an empty Node Set\n",
+                                                arg);
                             break;
                         case XPATH_BOOLEAN:
                             xmlGenericError(xmlGenericErrorContext,
@@ -2183,6 +2273,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
                         case XPATH_NODESET:{
                                 int indx;
 
+				if (list->nodesetval == NULL)
+				    break;
+
                                 for (indx = 0;
                                      indx < list->nodesetval->nodeNr;
                                      indx++) {
