@@ -27,6 +27,7 @@
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 #include <libxml/xmlerror.h>
+#include <libxml/globals.h>
 
 #ifdef LIBXML_XPTR_ENABLED
 
@@ -2027,7 +2028,7 @@ xmlXPtrInsideRange(xmlXPathParserContextPtr ctxt, xmlXPathObjectPtr loc) {
 					   xmlXPtrGetArity(node)));
 		}
 		default:
-		    return(NULL);
+		    break;
 	    }
 	    return(NULL);
 	}
@@ -2064,7 +2065,7 @@ xmlXPtrInsideRange(xmlXPathParserContextPtr ctxt, xmlXPathObjectPtr loc) {
 					       xmlXPtrGetArity(node)));
 		    }
 		    default:
-			return(NULL);
+			break;
 		}
 		return(NULL);
 	    }
@@ -2629,7 +2630,7 @@ xmlXPtrGetStartPoint(xmlXPathObjectPtr obj, xmlNodePtr *node, int *indx) {
 		*indx = obj->index;
 	    return(0);
 	default:
-	    return(-1);
+	    break;
     }
     return(-1);
 }
@@ -2665,7 +2666,7 @@ xmlXPtrGetEndPoint(xmlXPathObjectPtr obj, xmlNodePtr *node, int *indx) {
 		*indx = obj->index;
 	    return(0);
 	default:
-	    return(-1);
+	    break;
     }
     return(-1);
 }
